@@ -1,9 +1,9 @@
 /*
  * BSD 2-Clause License
  *
- * Copyright (c) 2014-2016, Lazaros Koromilas <lostd@2f30.org>
- * Copyright (c) 2014-2016, Dimitris Papastamos <sin@2f30.org>
- * Copyright (c) 2016-2019, Arun Prakash Jana <engineerarun@gmail.com>
+ * Copyright (C) 2014-2016, Lazaros Koromilas <lostd@2f30.org>
+ * Copyright (C) 2014-2016, Dimitris Papastamos <sin@2f30.org>
+ * Copyright (C) 2016-2019, Arun Prakash Jana <engineerarun@gmail.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -43,15 +43,22 @@ enum action {
 	SEL_PREV,
 	SEL_PGDN,
 	SEL_PGUP,
+	SEL_CTRL_D,
+	SEL_CTRL_U,
 	SEL_HOME,
 	SEL_END,
 	SEL_CDHOME,
 	SEL_CDBEGIN,
 	SEL_CDLAST,
+	SEL_CDROOT,
+	SEL_VISIT,
 	SEL_LEADER,
 	SEL_CYCLE,
+	SEL_CTX1,
+	SEL_CTX2,
+	SEL_CTX3,
+	SEL_CTX4,
 	SEL_PIN,
-	SEL_VISIT,
 	SEL_FLTR,
 	SEL_MFLTR,
 	SEL_TOGGLEDOT,
@@ -65,10 +72,13 @@ enum action {
 	SEL_FSIZE,  /* file size */
 	SEL_ASIZE,  /* apparent size */
 	SEL_BSIZE,  /* block size */
+	SEL_EXTN,   /* order by extension */
 	SEL_MTIME,
+	SEL_WILD,
 	SEL_REDRAW,
 	SEL_COPY,
 	SEL_COPYMUL,
+	SEL_COPYALL,
 	SEL_COPYLIST,
 	SEL_CP,
 	SEL_MV,
@@ -78,17 +88,22 @@ enum action {
 	SEL_NEW,
 	SEL_RENAME,
 	SEL_RENAMEALL,
+	SEL_SSHFS,
+	SEL_UMOUNT,
 	SEL_HELP,
 	SEL_EXEC,
 	SEL_SHELL,
-	SEL_SCRIPT,
+	SEL_PLUGIN,
+	SEL_LAUNCH,
 	SEL_RUNCMD,
 	SEL_RUNEDIT,
 	SEL_RUNPAGE,
+	SEL_NOTE,
 	SEL_LOCK,
 	SEL_QUITCTX,
 	SEL_QUITCD,
 	SEL_QUIT,
+	SEL_CLICK,
 };
 
 /* Associate a pressed key to an action */
@@ -99,12 +114,8 @@ struct key {
 
 static struct key bindings[] = {
 	/* Back */
-	{ KEY_BACKSPACE,  SEL_BACK },
-	{ '\b' /* BS */,  SEL_BACK },
-	{ 127 /* DEL */,  SEL_BACK },
 	{ KEY_LEFT,       SEL_BACK },
 	{ 'h',            SEL_BACK },
-	{ CONTROL('H'),   SEL_BACK },
 	/* Inside or select */
 	{ KEY_ENTER,      SEL_GOIN },
 	{ '\r',           SEL_GOIN },
@@ -114,43 +125,48 @@ static struct key bindings[] = {
 	/* Next */
 	{ 'j',            SEL_NEXT },
 	{ KEY_DOWN,       SEL_NEXT },
-	{ CONTROL('N'),   SEL_NEXT },
 	/* Previous */
 	{ 'k',            SEL_PREV },
 	{ KEY_UP,         SEL_PREV },
-	{ CONTROL('P'),   SEL_PREV },
 	/* Page down */
 	{ KEY_NPAGE,      SEL_PGDN },
-	{ CONTROL('D'),   SEL_PGDN },
 	/* Page up */
 	{ KEY_PPAGE,      SEL_PGUP },
-	{ CONTROL('U'),   SEL_PGUP },
+	/* Ctrl+D */
+	{ CONTROL('D'),   SEL_CTRL_D },
+	/* Ctrl+U */
+	{ CONTROL('U'),   SEL_CTRL_U },
 	/* First entry */
 	{ KEY_HOME,       SEL_HOME },
 	{ 'g',            SEL_HOME },
 	{ CONTROL('A'),   SEL_HOME },
-	{ '^',            SEL_HOME },
 	/* Last entry */
 	{ KEY_END,        SEL_END },
 	{ 'G',            SEL_END },
 	{ CONTROL('E'),   SEL_END },
-	{ '$',            SEL_END },
 	/* HOME */
 	{ '~',            SEL_CDHOME },
 	/* Initial directory */
-	{ '&',            SEL_CDBEGIN },
+	{ '@',            SEL_CDBEGIN },
 	/* Last visited dir */
 	{ '-',            SEL_CDLAST },
+	/* Go to / */
+	{ '`',            SEL_CDROOT },
+	/* Visit marked directory */
+	{ CONTROL('B'),   SEL_VISIT },
 	/* Leader key */
 	{ CONTROL('_'),   SEL_LEADER },
-	{ '`',            SEL_LEADER },
+	{ ',',            SEL_LEADER },
 	/* Cycle contexts in forward direction */
 	{ '\t',           SEL_CYCLE },
 	{ CONTROL('I'),   SEL_CYCLE },
+	/* Go to/create context N */
+	{ '1',            SEL_CTX1 },
+	{ '2',            SEL_CTX2 },
+	{ '3',            SEL_CTX3 },
+	{ '4',            SEL_CTX4 },
 	/* Mark a path to visit later */
 	{ 'b',            SEL_PIN },
-	/* Visit marked directory */
-	{ CONTROL('W'),   SEL_VISIT },
 	/* Filter */
 	{ '/',            SEL_FLTR },
 	/* Toggle filter mode */
@@ -178,16 +194,22 @@ static struct key bindings[] = {
 	{ 'S',            SEL_ASIZE },
 	/* Sort by total block count including dir contents */
 	{ CONTROL('J'),   SEL_BSIZE },
+	/* Sort by file extension */
+	{ 'E',            SEL_EXTN },
 	/* Toggle sort by time */
 	{ 't',            SEL_MTIME },
+	/* Wild load */
+	{ CONTROL('W'),   SEL_WILD },
 	/* Redraw window */
 	{ CONTROL('L'),   SEL_REDRAW },
 	/* Copy currently selected file path */
 	{ CONTROL('K'),   SEL_COPY },
 	{ ' ',            SEL_COPY },
 	/* Toggle copy multiple file paths */
+	{ 'K',            SEL_COPYMUL },
 	{ CONTROL('Y'),   SEL_COPYMUL },
-	{ 'Y',            SEL_COPYMUL },
+	/* Select all files in current dir */
+	{ 'Y',            SEL_COPYALL },
 	/* Show list of copied files */
 	{ 'y',            SEL_COPYLIST },
 	/* Copy from copy buffer */
@@ -204,8 +226,13 @@ static struct key bindings[] = {
 	{ 'n',            SEL_NEW },
 	/* Show rename prompt */
 	{ CONTROL('R'),   SEL_RENAME },
+	{ KEY_F(2),       SEL_RENAME }, // undocumented
 	/* Rename contents of current dir */
 	{ 'r',            SEL_RENAMEALL },
+	/* Connect to server over SSHFS */
+	{ 'c',            SEL_SSHFS },
+	/* Disconnect a SSHFS mount point */
+	{ 'u',            SEL_UMOUNT },
 	/* Show help */
 	{ '?',            SEL_HELP },
 	/* Execute file */
@@ -213,13 +240,18 @@ static struct key bindings[] = {
 	/* Run command */
 	{ '!',            SEL_SHELL },
 	{ CONTROL(']'),   SEL_SHELL },
-	/* Run a custom script */
-	{ 'R',            SEL_SCRIPT },
+	/* Run a plugin */
+	{ 'R',            SEL_PLUGIN },
+	{ CONTROL('V'),   SEL_PLUGIN },
+	/* Launcher */
+	{ '=',            SEL_LAUNCH },
 	/* Run a command */
-	{ CONTROL('S'),   SEL_RUNCMD },
+	{ CONTROL('P'),   SEL_RUNCMD },
 	/* Open in EDITOR or PAGER */
 	{ 'e',            SEL_RUNEDIT },
 	{ 'p',            SEL_RUNPAGE },
+	/* Open notes file */
+	{ CONTROL('N'),   SEL_NOTE },
 	/* Lock screen */
 	{ 'L',            SEL_LOCK },
 	/* Quit a context */
@@ -229,4 +261,5 @@ static struct key bindings[] = {
 	/* Quit */
 	{ 'Q',            SEL_QUIT },
 	{ CONTROL('Q'),   SEL_QUIT },
+	{ KEY_MOUSE,      SEL_CLICK },
 };
